package com.retmix.volga.shared.services.impl;

import com.retmix.volga.shared.dto.*;
import com.retmix.volga.shared.dto.admin.UpsertUserDTO;
import com.retmix.volga.shared.handler.exceptions.TokenInvalidException;
import com.retmix.volga.shared.handler.exceptions.UserNotFoundException;
import com.retmix.volga.shared.models.User;
import com.retmix.volga.shared.repositories.UserRepository;
import com.retmix.volga.shared.services.UserService;
import com.retmix.volga.storage.models.Directory;
import com.retmix.volga.storage.repositories.DirectoryRepository;
import com.retmix.volga.utils.PasswordHash;
import com.retmix.volga.utils.jwt.JwtUtil;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.security.NoSuchAlgorithmException;

@Service
@AllArgsConstructor
public class UserServiceImpl implements UserService {
    private final UserRepository userRepository;
    private final JwtUtil jwtService;
    private final PasswordHash passwordHash;
    private final DirectoryRepository directoryRepository;

    @Override
    public EnterDTO signUp(UpsertUserDTO data) throws NoSuchAlgorithmException {
        User user = new User();
        user.setEmail(data.getEmail());
        user.setPassword(passwordHash.hashPassword(data.getPassword()));
        String token = jwtService.generateToken(user.toDTO());
        user.setToken(token);
        User userSave = userRepository.save(user);
        Directory dir = Directory.builder()
                .name("root")
                .root(true)
                .user(user).build();
        directoryRepository.save(dir);

        return user.toEnterDTO();
    }

    @Override
    public MessageDTO logout(User user) {
        user.setToken(null);
        userRepository.save(user);
        return new MessageDTO("logout");
    }

    @Override
    public UserDTO show(User user) {
        return user.toDTO();
    }

    @Override
    public UserDTO update(UpdatePassword data, User user) throws NoSuchAlgorithmException {
        user.setPassword(passwordHash.hashPassword(data.getPassword()));
        return userRepository.save(user).toDTO();
    }

    @Override
    public EnterDTO login(LoginDTO data) throws NoSuchAlgorithmException {
        User user = userRepository.findUserByEmail(data.getEmail())
                .orElseThrow(() -> new UserNotFoundException("Invalid username or password"));
        if (!passwordHash.checkPassword(data.getPassword(), user.getPassword())) {
            throw new TokenInvalidException("Invalid password or username");
        }
        String token = jwtService.generateToken(user.toDTO());
        user.setToken(token);
        userRepository.save(user);
        return user.toEnterDTO();
    }

    @Override
    public User getUserByToken(String token) {
        String tokenPrepare = prepareToken(token);
        return userRepository.findUserByToken(tokenPrepare).orElseThrow(() -> new RuntimeException("User not found"));
    }

    private String prepareToken(String token) {
        if (token == null) {
            throw new TokenInvalidException("Invalid token");
        }
        return token.substring(token.indexOf(" ")).trim();
    }
}